﻿using System.Security.Cryptography.X509Certificates;
using FitConnect;
using FitConnect.Models;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;

var clientId = "";
var clientSecret = "";
PublicKey publicKey;
ILogger _logger;
Client client;
X509Certificate2 certificate;

/*
 * The easy way to call the FitConnect API
 */
async Task AbstractCall() {
    client = new Client(
        FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
        "clientId", "clientSecret");

    var success = await client.SendSubmissionAsync(new Submission());

    var submissions = await client.GetSubmissionsAsync("destinationId");

    foreach (var submission in submissions) Console.WriteLine(submission.Id);
}


void FluentSenderCall() {
    client.Sender
        .Authenticate(clientId!, clientSecret!)
        .CreateSubmission(new Submission { Attachments = new List<Attachment>() })
        .UploadAttachments()
        .SendSubmission(new Metadata(), new Data());
}

void FluentSubscriberCall() {
    client.Subscriber
        // Polling available submissions
        .Authenticate(clientId!, clientSecret!)
        .PollSubmissions("destinationId", out var _)
        // 
        // Getting list of submissions
        .Authenticate(clientId!, clientSecret!)
        .GetSubmissions("destinationId", out var _)
        //
        // Reading submission
        .Authenticate(clientId!, clientSecret!)
        .GetSubmission("destinationId", "submissionId", out var _)
        .GetAttachments(out var _)
        .DecryptAttachments(out var _)
        .DecryptData(out var _)
        .DecryptMetadata(out var _)
        .ConfirmSubmission();
}

/*
 * The more verbose way to call the FitConnect API
 * Here to call the **Sender** interface
 */
async Task DetailSenderCall() {
    var sender =
         Sender.Create(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development), certificate);

    var submissionDto = sender.AddSubmission(new Submission());
    var encryptedAttachments = sender.Encrypt(publicKey, new List<Attachment>());
    var success = await sender.UploadAttachmentsAsync(encryptedAttachments);
    var encryptedMetadata = sender.Encrypt(publicKey, new Metadata());
    var encryptedData = sender.Encrypt(publicKey, new Data());
    success = await sender.SubmitAsync(encryptedData, encryptedMetadata);
}

/*
 * The more verbose way to call the FitConnect API
 * Here to call the **Subscriber** interface
 */

async Task DetailSubscriberCall() {
    var subscriber =
        Subscriber.Create(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development), certificate);
    var submissions = await subscriber.GetSubmissionsAsync("destinationId");
    /*
     ....
     */
}


Console.WriteLine(
    "This is a dummy client to demonstrate the usage of the FitConnect SDK for .NET");

_logger = new Microsoft.Extensions.Logging.Logger<System.AppDomain>(new NullLoggerFactory());
client =
    new Client(FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
        logger: _logger);
